/**
 * @file
 * Provides GridStack admin base utilities.
 */

(function ($, Drupal, _, _db, window) {

  'use strict';

  var GRIDSTACK_DEFAULT_GRIDS = {
    x: 0,
    y: 0,
    w: 2,
    h: 2
  };

  Drupal.gridstack = Drupal.gridstack || {};

  /**
   * The GridStack admin base.
   */
  Drupal.gridstack.admin = {
    config: {},
    options: {},
    getDefaultBox: function () {
      return GRIDSTACK_DEFAULT_GRIDS;
    },

    getModelId: function ($box) {
      $box = $box.length ? $box : $($box);
      if ($box.length) {
        // Nested boxes are not backbone model, looking for the box model ID.
        var $nested = $box.closest('.gridstack--nested');
        return $nested.length ? $nested.data('gsBid') : $box.data('gsBid');
      }
      return -1;
    },

    lastBoxIndex: 0,

    isValidNode: function (node) {
      return !_.isNull(node) && !_.isUndefined(node) && (!_.isUndefined(node.x) && !_.isUndefined(node.w));
    },

    node: function (node) {
      var me = this;
      var box = {};

      if (me.isValidNode(node)) {
        box = {
          x: parseInt(node.x),
          y: parseInt(node.y),
          w: parseInt(node.w),
          h: parseInt(node.h)
        };

        if (!_.isEmpty(node.region)) {
          box.region = node.region;
        }
      }
      return box;
    },

    attributes: function (o) {
      o = o || {};

      var attributes = o;
      if (_.isUndefined(o.index)) {
        return attributes;
      }

      var index = o.index;
      attributes.id = index;

      if (!_.isUndefined(o.indexNested)) {
        var indexNested = o.indexNested;

        attributes.mid = attributes.id;
        attributes.mindex = index;

        attributes.id = index + '_' + indexNested;
        attributes.index = indexNested;
      }
      else {
        attributes.index = index;
      }

      return attributes;
    },

    updateBoxDimensions: function (box, node) {
      var $box = box.length ? box : $(box);

      node = node || $box[0].gridstackNode;
      if (this.isValidNode(node)) {
        $box.attr('data-gs-dimension', node.w + 'x' + node.h);
      }
    },

    oldData: function (el, isNested) {
      isNested = isNested === 'undefined' ? false : isNested;

      return el.data(isNested ? 'nestedGrids' : 'previewGrids') || [];
    },

    newData: function (el, isNested) {
      isNested = isNested === 'undefined' ? false : isNested;

      var me = this;
      var storage = el.data(isNested ? 'nestedStorage' : 'storage');
      var $storage = $('[data-drupal-selector="' + storage + '"]');
      var val = $storage.length ? $storage.val() : '';
      var newData = _.isEmpty(val) || val === '[]' ? '' : val;

      if (newData) {
        newData = _db.parse(newData) || [];
      }

      // Ensure data can be reverted since dataStored is changing on save.
      var data = newData || me.oldData(el, isNested) || [];

      // Might happen when all nodes are deleted, add one in the least on load.
      return data.length ? data : [GRIDSTACK_DEFAULT_GRIDS];
    }

  };

})(jQuery, Drupal, _, dBlazy, this);
